package dev.shadowsoffire.apotheosis.adventure.affix.salvaging;

import javax.annotation.Nullable;
import dev.shadowsoffire.apotheosis.Apoth;
import java.util.ArrayList;
import java.util.Collections;
import net.minecraft.world.level.Level;
import net.minecraft.util.RandomSource;
import java.util.Iterator;
import java.util.List;
import net.minecraft.server.level.ServerPlayer;
import net.minecraft.sounds.SoundSource;
import net.minecraft.sounds.SoundEvents;
import net.minecraft.world.Container;
import net.minecraftforge.items.IItemHandlerModifiable;
import net.minecraftforge.items.wrapper.RecipeWrapper;
import dev.shadowsoffire.placebo.menu.FilteredSlot;
import com.google.common.base.Predicates;
import net.minecraft.world.inventory.Slot;
import net.minecraft.world.item.ItemStack;
import java.util.function.Predicate;
import dev.shadowsoffire.placebo.menu.PlaceboContainerMenu;
import dev.shadowsoffire.apotheosis.adventure.Adventure;
import net.minecraft.world.inventory.MenuType;
import net.minecraft.core.BlockPos;
import net.minecraft.world.entity.player.Inventory;
import dev.shadowsoffire.placebo.cap.InternalItemHandler;
import net.minecraft.world.entity.player.Player;
import dev.shadowsoffire.placebo.menu.BlockEntityMenu;

public class SalvagingMenu extends BlockEntityMenu<SalvagingTableTile>
{
    protected final Player player;
    protected final InternalItemHandler inputInv;
    
    public SalvagingMenu(final int id, final Inventory inv, final BlockPos pos) {
        super((MenuType)Adventure.Menus.SALVAGE.get(), id, inv, pos);
        this.inputInv = new InternalItemHandler(12);
        this.player = inv.f_35978_;
        int leftOffset = 17;
        int topOffset = 17;
        for (int i = 0; i < 12; ++i) {
            this.m_38897_((Slot)new PlaceboContainerMenu.UpdatingSlot(this.inputInv, i, leftOffset + i % 4 * 19, topOffset + i / 4 * 19, s -> findMatch(this.level, s) != null) {
                public int m_6641_() {
                    return 1;
                }
                
                public int m_5866_(final ItemStack stack) {
                    return 1;
                }
            });
        }
        for (int i = 0; i < 6; ++i) {
            leftOffset = 124;
            topOffset = 17;
            this.m_38897_((Slot)new FilteredSlot(((SalvagingTableTile)this.tile).output, i, leftOffset + i % 2 * 19, topOffset + i / 2 * 19, (Predicate)Predicates.alwaysFalse()));
        }
        this.addPlayerSlots(inv, 8, 92);
        this.mover.registerRule((stack, slot) -> slot >= this.playerInvStart && findMatch(this.level, stack) != null, 0, 12);
        this.mover.registerRule((stack, slot) -> slot < this.playerInvStart, this.playerInvStart, this.hotbarStart + 9);
        this.registerInvShuffleRules();
    }
    
    public boolean m_6875_(final Player player) {
        return this.level.f_46443_ || this.level.m_8055_(this.pos).m_60734_() == Adventure.Blocks.SALVAGING_TABLE.get();
    }
    
    public void m_6877_(final Player player) {
        super.m_6877_(player);
        if (!this.level.f_46443_) {
            this.m_150411_(player, (Container)new RecipeWrapper((IItemHandlerModifiable)this.inputInv));
        }
    }
    
    public boolean m_6366_(final Player player, final int id) {
        if (id == 0) {
            this.salvageAll();
            this.level.m_5594_((Player)null, player.m_20183_(), SoundEvents.f_11862_, SoundSource.BLOCKS, 0.99f, this.level.f_46441_.m_188501_() * 0.25f + 1.0f);
            this.level.m_5594_((Player)null, player.m_20183_(), SoundEvents.f_144054_, SoundSource.BLOCKS, 0.34f, this.level.f_46441_.m_188501_() * 0.2f + 0.8f);
            this.level.m_5594_((Player)null, player.m_20183_(), SoundEvents.f_12471_, SoundSource.BLOCKS, 0.45f, this.level.f_46441_.m_188501_() * 0.5f + 0.75f);
            return true;
        }
        return super.m_6366_(player, id);
    }
    
    protected void giveItem(final Player player, final ItemStack stack) {
        if (!player.m_6084_() || (player instanceof ServerPlayer && ((ServerPlayer)player).m_9232_())) {
            player.m_36176_(stack, false);
        }
        else {
            final Inventory inventory = player.m_150109_();
            if (inventory.f_35978_ instanceof ServerPlayer) {
                inventory.m_150079_(stack);
            }
        }
    }
    
    protected void salvageAll() {
        for (int inSlot = 0; inSlot < 12; ++inSlot) {
            final Slot s = this.m_38853_(inSlot);
            final ItemStack stack = s.m_7993_();
            final List<ItemStack> outputs = salvageItem(this.level, stack);
            s.m_5852_(ItemStack.f_41583_);
            for (ItemStack out : outputs) {
                for (int outSlot = 0; outSlot < 6 && !out.m_41619_(); out = ((SalvagingTableTile)this.tile).output.insertItem(outSlot, out, false), ++outSlot) {}
                if (!out.m_41619_()) {
                    this.giveItem(this.player, out);
                }
            }
        }
    }
    
    public static int getSalvageCount(final SalvagingRecipe.OutputData output, final ItemStack stack, final RandomSource rand) {
        final int[] counts = getSalvageCounts(output, stack);
        return rand.m_216339_(counts[0], counts[1] + 1);
    }
    
    public static int[] getSalvageCounts(final SalvagingRecipe.OutputData output, final ItemStack stack) {
        final int[] out = { output.min, output.max };
        if (stack.m_41763_()) {
            out[1] = Math.max(out[0], Math.round((float)(out[1] * (stack.m_41776_() - stack.m_41773_()) / stack.m_41776_())));
        }
        return out;
    }
    
    public static List<ItemStack> salvageItem(final Level level, final ItemStack stack) {
        final SalvagingRecipe recipe = findMatch(level, stack);
        if (recipe == null) {
            return Collections.emptyList();
        }
        final List<ItemStack> outputs = new ArrayList<ItemStack>();
        for (final SalvagingRecipe.OutputData d : recipe.getOutputs()) {
            final ItemStack out = d.stack.m_41777_();
            out.m_41764_(getSalvageCount(d, stack, level.f_46441_));
            outputs.add(out);
        }
        return outputs;
    }
    
    public static List<ItemStack> getBestPossibleSalvageResults(final Level level, final ItemStack stack) {
        final SalvagingRecipe recipe = findMatch(level, stack);
        if (recipe == null) {
            return Collections.emptyList();
        }
        final List<ItemStack> outputs = new ArrayList<ItemStack>();
        for (final SalvagingRecipe.OutputData d : recipe.getOutputs()) {
            final ItemStack out = d.stack.m_41777_();
            out.m_41764_(getSalvageCounts(d, stack)[1]);
            outputs.add(out);
        }
        return outputs;
    }
    
    @Nullable
    public static SalvagingRecipe findMatch(final Level level, final ItemStack stack) {
        for (final SalvagingRecipe recipe : level.m_7465_().m_44013_(Apoth.RecipeTypes.SALVAGING)) {
            if (recipe.matches(stack)) {
                return recipe;
            }
        }
        return null;
    }
}
